{Version 1.05 21/8/98}
unit Jrebzbtn;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TJREBezelBtn = class(TCustomPanel)
  private
    FOnBezelColour: TColor;
    FOffBezelColour: TColor;
    FOnCaption: TCaption;
    FOffCaption: TCaption;
    FPressed: boolean;
    FLatched: boolean;
    FColor: TColor;
    FOnChange: TNotifyEvent;
  protected
    EdgeSize: integer;
    procedure SetName(const NewName: TComponentName); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure DoChange; virtual;
    procedure SetOnBezelColour(NewVal: TColor);
    procedure SetOffBezelColour(NewVal: TColor);
    procedure SetOnCaption(NewVal: TCaption);
    procedure SetOffCaption(NewVal: TCaption);
    procedure SetPressed(NewVal: boolean);
    procedure SetLatched(NewVal: boolean);
    procedure SetColor(NewVal: TColor);
    property BevelInner default bvRaised;
    property BevelOuter default bvLowered;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click(Sender: TObject);
  published
    property Width default 112;
    property Height default 40;
    property BevelWidth;
    property BorderStyle;
    property BorderWidth default 4;
    property Visible;
    property TabStop default true;
    property TabOrder;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEnter;
    property OnExit;
    property OnKeyPress;
    property OnKeyDown;
    property OnKeyUp;
    property Font;
    property ParentFont;
    property Color: TColor read FColor write SetColor default clBtnFace;
    property OnBezelColour: TColor read FOnBezelColour write SetOnBezelColour default clRed;
    property OffBezelColour: TColor read FOffBezelColour write SetOffBezelColour default clGray;
    property OnCaption: TCaption read FOnCaption write SetOnCaption;
    property OffCaption: TCaption read FOffCaption write SetOffCaption;
    property Pressed: boolean read FPressed write SetPressed default false;
    property Latched: boolean read FLatched write SetLatched default false;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TJREBezelBtn]);
end;

constructor TJREBezelBtn.Create(AOwner: TComponent);
begin
  try
    inherited Create(AOwner);
    Width := 112;
    Height := 40;
    BevelInner := bvRaised;
    BevelOuter := bvLowered;
    BorderWidth := 4;
    TabStop := true;
    FColor := clBtnFace;
    FOnBezelColour := clRed;
    FOffBezelColour := clGray;
    FOffCaption := 'JustCreated';
    FOnCaption := '';
    FPressed := false;
    FLatched := false;
  except
    MessageBox(0, 'Unable to create button: closing down program',
                      'TJREBezelBtn.Create function', mb_OK);
    Application.Terminate;
  end;
end;

procedure TJREBezelBtn.SetName(const NewName: TComponentName);
begin
  if (FOffCaption = 'JustCreated') or (FOffCaption = Name) then begin
    OffCaption := NewName;
    OnCaption := NewName;
  end;
  inherited SetName(NewName);
  Caption := '';
end;

procedure TJREBezelBtn.Paint;
var
  CapText: string;
begin
  if Pressed then begin
    CapText := OnCaption;
    inherited Color := OnBezelColour;
  end
  else begin
    CapText := OffCaption;
    inherited Color := OffBezelColour;
  end;
  inherited Paint;
  if BorderStyle = bsNone then EdgeSize := 0 else EdgeSize := 1;
  if BevelOuter <> bvNone then EdgeSize := EdgeSize + BevelWidth;
  if BevelInner <> bvNone then EdgeSize := EdgeSize + BevelWidth + BorderWidth;
  Canvas.Brush.Color := Color;
  Canvas.Pen.Width := 1;
  if Focused then begin
    Canvas.Pen.Color := clBlack;
  end
  else begin
    Canvas.Pen.Color := Color;
  end;
  Canvas.Rectangle(EdgeSize, EdgeSize, Width - EdgeSize, Height - EdgeSize);
  Canvas.Pen.Style := psSolid;
  if Pressed then CapText := OnCaption else CapText := OffCaption;
  if BevelInner = bvLowered then
    Canvas.TextOut((Width - Canvas.TextWidth(CapText)) div 2 + BevelWidth,
                  (Height - Canvas.TextHeight(CapText)) div 2 + BevelWidth, CapText)
  else
    Canvas.TextOut((Width - Canvas.TextWidth(CapText)) div 2,
                  (Height - Canvas.TextHeight(CapText)) div 2, CapText);
end;

procedure TJREBezelBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  BevelInner := bvLowered;
  Invalidate;
end;

procedure TJREBezelBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (not Pressed) or (not Latched) then Pressed := not Pressed;
  inherited MouseUp(Button, Shift, X, Y);
  BevelInner := bvRaised;
end;

procedure TJREBezelBtn.Click(Sender: TObject);
begin
  inherited Click;
  if (not Pressed) or (not Latched) then Pressed := not Pressed;
end;

procedure TJREBezelBtn.DoEnter;
begin
  Invalidate;
  inherited DoEnter;
end;

procedure TJREBezelBtn.DoExit;
begin
  Invalidate;
  inherited DoExit;
end;

procedure TJREBezelBtn.KeyDown(var Key: Word; Shift: TShiftState);
begin
  Pressed := not Pressed;
  inherited KeyDown(Key, Shift);
end;

procedure TJREBezelBtn.SetOnBezelColour(NewVal: TColor);
begin
  if NewVal <> OnBezelColour then begin
    FOnBezelColour := NewVal;
    Invalidate;
  end;
end;

procedure TJREBezelBtn.SetOffBezelColour(NewVal: TColor);
begin
  if NewVal <> OffBezelColour then begin
    FOffBezelColour := NewVal;
    Invalidate;
  end;
end;

procedure TJREBezelBtn.SetOnCaption(NewVal: TCaption);
begin
  if NewVal <> OnCaption then begin
    FOnCaption := NewVal;
    Invalidate;
  end;
end;

procedure TJREBezelBtn.SetOffCaption(NewVal: TCaption);
begin
  if NewVal <> OffCaption then begin
    FOffCaption := NewVal;
    Invalidate;
  end;
end;

procedure TJREBezelBtn.SetPressed(NewVal: boolean);
begin
  if NewVal <> Pressed then begin
    FPressed := NewVal;
    Invalidate;
    DoChange;
  end;
end;

procedure TJREBezelBtn.SetLatched(NewVal: boolean);
begin
  if NewVal <> Latched then begin
    FLatched := NewVal;
  end;
end;

procedure TJREBezelBtn.SetColor(NewVal: TColor);
begin
  if NewVal <> Color then begin
    FColor := NewVal;
    Invalidate;
  end;
end;

procedure TJREBezelBtn.DoChange;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

end.
{Version history
1.04 First released version: DSP 1/8/98
1.05 Add Latched property
Port to Delphi3 23/8/98
}
